/**
 *
 * Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.apache.geronimo.security.remoting;

import org.apache.geronimo.security.jaas.LoginServiceMBean;


/**
 * @version $Revision: 1.2 $ $Date: 2004/02/25 09:58:10 $
 */
abstract public class RemoteLoginServiceFactory {

    private static RemoteLoginServiceFactory instance = createRemoteLoginServiceFactory();

    /**
     * @param hostname
     * @return
     */
    public static LoginServiceMBean create(String hostname) {
        return instance.factoryCreate(hostname);
    }

    /**
     * @param hostname
     */
    abstract protected LoginServiceMBean factoryCreate(String hostname);

    /**
     * @return
     */
    private static RemoteLoginServiceFactory createRemoteLoginServiceFactory() {
        try {
            // Get the factory name via sys prop...
            // In case we EVER used a different factory than: org.apache.geronimo.security.remoting.jmx.RemoteLoginServiceFactory
            String factoryClass = System.getProperty("org.apache.geronimo.security.remoting.RemoteLoginServiceFactory", "org.apache.geronimo.security.remoting.jmx.RemoteLoginServiceFactory");
            return (RemoteLoginServiceFactory) RemoteLoginServiceFactory.class.getClassLoader().loadClass(factoryClass).newInstance();
        } catch (Throwable e) {
            throw new RuntimeException("The RemoteLoginServiceFactory instance could not be loaded:", e);
        }
    }
}
